# Copyright 2017 ZTE, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
'''
Created on 2017年9月21日

@author: 10169294
'''

TAIL_NUM = 5


class CmccUtil(object):

    def filter_gw_port(self, op_version, **kwargs):
        if op_version == "bigcloud_K":
            if kwargs['router']['name'][-TAIL_NUM:] == '_CORE':
                kwargs["gw_port"] = ""
        else:
            return kwargs["gw_port"]

        return kwargs["gw_port"]
